﻿using System;
using System.Runtime.InteropServices;    // For StructLayout, DllImport

namespace WhaleTeqPPG_SDK
{
    public enum OutputFunction
    {
        Output_Off = 0,
        Output_Sine,
        Output_Triangle,
        Output_Square,
        Output_PPG = 15,
        Output_InvPPG = 16
    };

    class PPG_SDK
    {
        //First API must be called. Return false for connected device not
        //found. "Hot Plug" support with "connCB" parameter informing
        //device connecting and un-connecting.
        [DllImport(@"WhaleTeqPPG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool InitPPG(IntPtr connCB);
        //Parameter - connCB:
        //  function pointer of delegate from "Marshal.GetFunctionPointerForDelegate(delegate)"
        //      delegate void ConnectCBDelegate(bool connected);
        //      Parameter - connected
        //	        TRUE: device is connected
        //	        FALSE: device is unplugged

        //Same as InitPPG() with connCB as NULL.
        //First API for "Hot Plug" unsupported like LabView environment
        [DllImport(@"WhaleTeqPPG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool StartPPG(IntPtr connCB);


        //Last calling API when system exiting for releasing resource
        [DllImport(@"WhaleTeqPPG_SDK.dll")]
        public static extern void ClosePPG();


        //Check HRS device is connected
        [DllImport(@"WhaleTeqPPG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool IsConnected();


        //Check HRS device wider PD range is supported
        [DllImport(@"WhaleTeqPPG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool IsWiderPDRange();


        //Register callback to get real time PD sampling data.
        //About 0.00003 second(30us) interval between each 2 sampling data
        [DllImport(@"WhaleTeqPPG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void RegisterPDSamplingCB(IntPtr cb);
        //Parameter - cb:
        //  function pointer of delegate from "Marshal.GetFunctionPointerForDelegate(delegate)"
        //      delegate void PDSamplingCBDelegate(int ValueNum, IntPtr ValueArray);
        //      Parameter - ValueNum
        //	        Number of sampling values in ValueArray
        //      Parameter - ValueArray
        //	        Sampling values array in unsigned short type


        //Register callback to get real time PD sampling Maximum Peak value in cycle seconds.
        [DllImport(@"WhaleTeqPPG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void RegisterPDSamplingPeakCB(IntPtr cb, double cycleSecond);
        //Parameter - cb:
        //  function pointer of delegate from "Marshal.GetFunctionPointerForDelegate(delegate)"
        //      delegate void PDSamplingPeakCBDelegate(int PeakValue);
        //      Parameter - PeakValue
        //	        The PD sampling Maximum Peak value in cycle seconds.
        //Parameter - cycleSecond
        //      The duration of each cycle Maximum Peak detection


        //Control PD Sampling functionality.
        [DllImport(@"WhaleTeqPPG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool SetSampleSetting(bool Enable, bool HighResolution);
        //Parameter - Enable
        //       TRUE to enable HRS PD Sampling
        //Parameter - HighResolution
        //       TRUE to turn on "HighResolution" mode. It only can be set when
        //       LED Off(by "SendLedOff()").


        //Specify PD Sampling Channel
        [DllImport(@"WhaleTeqPPG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void SetSampleChannel(int Channel);
        //Parameter - Channel
        //       0 as Channel 1 (default setting)
        //       1 as Channel 2


        //Turn off LED(testing signal sending) functionality.
        [DllImport(@"WhaleTeqPPG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool SendLedOff();


        [DllImport(@"WhaleTeqPPG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool SendCommand(int DCLevel, float PI, int HR, bool SyncPD);
        //Parameter - DCLevel
        //      Range: 1 ~ 20
        //Parameter - PI
        //      Range changed corresponding to DCLevel:
        //      MaxPI = 20.0 / DCLevel
        //      MinPI = 3.0 / 5 * DCLevel
        //Parameter - HR
        //      Range: 30~300 bpm
        //Parameter - SyncPD
        //      TRUE for testing signal sending synchronized with PD receiving signal


        //Register callback to get real time output signal data in "FineTune" mode.
        //About 0.006 second(6ms) interval between each callback be triggered
        [DllImport(@"WhaleTeqPPG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void RegisterTimeAmpCB(IntPtr cb);
        //Parameter - cb:
        //  function pointer of delegate from "Marshal.GetFunctionPointerForDelegate(delegate)"
        //      delegate void TimeAmplitudeCBDelegate(double time, double amp);
        //      Parameter - time
        //	        unit: second
        //	        time from the output signal changed(function, amplitude, frequency ...)
        //      Parameter - amp
        //	        unit: microvolt


        //Sending testing signal with specified Waveform Type, Noise Hz, DC Value, AC Value and Heart Rate(BPM)
        [DllImport(@"WhaleTeqPPG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool SendFineTune(OutputFunction Waveform, int NoiseHz, int DC, int AC, int BPM, bool SyncPD);
        //Parameter - Waveform
        //      Testing signal waveform type
        //Parameter - NoiseHz
        //      Value can be 0(without Noize), 50, 60 or 1k Hz
        //Parameter - DC
        //      Range: 100 ~ 2500 mV
        //Parameter - AC
        //      Range: 75 ~ 2500 uV (means 0.75 ~ 25 mV)
        //Parameter - BPM
        //      Range: 30 ~ 300 bpm
        //Parameter - SyncPD
        //      TRUE for testing signal sending synchronized with PD receiving signal
        //Return value:
        //      false for HRS device not connected or Module HRS100 connected



        //Set the Noise Amplitude in "FineTune" Mode
        [DllImport(@"WhaleTeqPPG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void SendFineTuneNoiseAmp(double Amp);
        //Parameter - Amp
        //       Default value 0.1 mV
        //       Range 0.01 ~ 0.2 mV


        //Load "Raw Data File" for dynamic playing. 
        //When succeed, it would return the sample rate, maximum value, minimum value and total time
        //with reference parameters.
        [DllImport(@"WhaleTeqPPG_SDK.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        public static extern int LoadDynamic(string File, ref double SampleRate, ref double Max, ref double Min, ref double StopTime);
        //Parameter - File
        //       Full path including file name of "Raw Data File".
        //Parameter - SampleRate
        //       The Sample Rate in "Raw Data File".
        //Parameter - Max
        //       The maximum raw data value in "Raw Data File".
        //Parameter - Min
        //       The minimum raw data value in "Raw Data File".
        //Parameter - StopTime
        //       The total sampling seconds in "Raw Data File".
        //Return value:
        // >0: succeed, the number of sample data loaded
        // -1: Open file failed
        // -2: Can't solve sample rate in Line 1
        // -3: Can't solve sample number in Line 2
        // -4: Raw data file is too large to fit the memory
        // -5: Unsupported API for HRS100 Module


        //Get loaded Raw Data value array.
        [DllImport(@"WhaleTeqPPG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr GetDynamicArray(ref int num);
        //Parameter - num
        //       The number of raw data values in ValueArray
        //Return value:
        //	     The raw data values array in double type


        //Start dynamic playing loaded Raw Data
        [DllImport(@"WhaleTeqPPG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        [return: MarshalAs(UnmanagedType.I1)]
        public static extern bool PlayDynamic(double F, int Offset, IntPtr stopCB, IntPtr playCB, bool SyncPD);
        //Parameter - Gain
        //	Default value is 4095.0 / (Max - Min)
        //Parameter - Offset
        //      Defalut value is "-Min" to normalize the raw data value range
        //Parameter - stopCB
        //      SDK supports "Loop" playing as default mode with stopCB as "(IntPtr)0";
        //      Specified "informing finish callback" to implement "non-Loop" playing mode;
        //      function pointer of delegate from "Marshal.GetFunctionPointerForDelegate(delegate)"
        //      delegate void PlayStopCBDelegate(void);
        //Parameter - playCB
        //      Specified real time playing data callback
        //      function pointer of delegate from "Marshal.GetFunctionPointerForDelegate(delegate)"
        //      delegate void PlayDynamicCBDelegate(double time, unsigned short DC, unsigned short AC);
        //Parameter - SyncPD
        //      TRUE for testing signal sending synchronized with PD receiving signal


        //Change the sample rate of loaded Raw Data for playing
        [DllImport(@"WhaleTeqPPG_SDK.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void ResampleDynamic(double newSampleRate, ref double stopTime);
        //Parameter - newSampleRate
        //      New Sample Rate value
        //Parameter - ref stopTime
        //      Updated totally playing time for Sample Rate changed
    }
}
