// WhaleTeqPPG_SDK_release.h

#ifdef WHALETEQPPG_SDK_EXPORTS
#define WHALETEQPPG_SDK_API __declspec(dllexport) 
#else
#define WHALETEQPPG_SDK_API __declspec(dllimport) 
#endif

extern "C"{
	//Callback support PPG device "Hot Plug" connecting and un-connecting
	typedef void (__stdcall *DeviceConnectedCB)(bool DeviceConnectedCB);
	//Parameter - connected
	//	TRUE: device is connected
	//	FALSE: device is unplugged

	//First API must be called. Return false for connected device not
	//found. "Hot Plug" support with "connCB" parameter informing
	//device connecting and un-connecting.
	WHALETEQPPG_SDK_API bool InitPPG(DeviceConnectedCB connCB);

	//Same as InitPPG() with connCB as NULL.
	//First API for "Hot Plug" unsupported like LabView environment
	WHALETEQPPG_SDK_API bool StartPPG();

	//Last calling API when system exiting for releasing resource
	WHALETEQPPG_SDK_API void ClosePPG();

	//Check HRS device is connected
	WHALETEQPPG_SDK_API bool IsConnected();

	//Check HRS device wider PD range is supported
	WHALETEQPPG_SDK_API bool IsWiderPDRange();

	//Callback for real time PD Sampling
	typedef void (__stdcall *PDSamplingCB)(int ValueNum, unsigned short* ValueArray);
	//Parameter - ValueNum
	//	Number of sampling values in ValueArray
	//Parameter - ValueArray
	//	Sampling values array

	//Register callback to get real time PD sampling data.
	//About 0.00003 second(30us) interval between each 2 sampling data
	WHALETEQPPG_SDK_API void RegisterPDSamplingCB(PDSamplingCB cb);

	//Callback for real time PD Sampling Peak value
	typedef void (__stdcall *PDSamplingPeakCB)(int PeakValue);
	//Parameter - PeakValue
	//	Maximum value of PD sampling in specified cycle time

	//Register callback to get real time PD sampling Maximum Peak value in cycle seconds.
	WHALETEQPPG_SDK_API void RegisterPDSamplingPeakCB(PDSamplingPeakCB cb, double cycleSecond);
	//Parameter - cycleSecond
	//      The duration of each cycle Maximum Peak detection

	//Control PD Sampling functionality.
	WHALETEQPPG_SDK_API bool SetSampleSetting(bool Enable, bool HighResolution);
	//Parameter - Enable
	//       TRUE to enable HRS PD Sampling
	//Parameter - HighResolution
	//       TRUE to turn on "HighResolution" mode. It only can be set when
	//       LED Off(by "SendLedOff()").

	//Specify PD Sampling Channel
	WHALETEQPPG_SDK_API void SetSampleChannel(int Channel);
	//Parameter - Channel
	//       0 as Channel 1 (default setting)
	//       1 as Channel 2

	//Turn off LED(testing signal sending) functionality.
	WHALETEQPPG_SDK_API bool SendLedOff();

	//Sending testing signal with specified DC Level, PI Value and Heart Rate(BPM)
	WHALETEQPPG_SDK_API bool SendCommand(int DCLevel, float PI, int HR, bool SyncPD);
	//Parameter - DCLevel
	//      Range: 1 ~ 20
	//Parameter - PI
	//      Range changed corresponding to DCLevel:
	//      MaxPI = 20.0 / DCLevel
	//      MinPI = 3.0 / 5 * DCLevel
	//Parameter - HR
	//      Range: 30 ~ 300 bpm
	//Parameter - SyncPD
	//      TRUE for testing signal sending synchronized with PD receiving signal

	//Callback for real time output signal amplitude
	typedef void (__stdcall *TimeAmplitudeCB)(double time, double amp);
	//Parameter - time
	//	unit: second
	//	time from the output signal changed(function, amplitude, frequency ...)
	//Parameter - amp
	//	unit: microvolt

	//Register callback to get real time output signal data in "FineTune" mode.
	//About 0.006 second(6ms) interval between each callback be triggered
	WHALETEQPPG_SDK_API void RegisterTimeAmpCB(TimeAmplitudeCB cb);

	//Sending testing signal with specified Waveform Type, Noise Hz, DC Value, AC Value and Heart Rate(BPM)
	enum OutputFunction_E {
		Output_Off	= 0,
		Output_Sine,
		Output_Triangle,
		Output_Square,
        Output_PPG = 15,
		Output_InvPPG = 16
	};
	WHALETEQPPG_SDK_API bool SendFineTune(OutputFunction_E Waveform, int NoiseHz, int DC, int AC, int BPM, bool SyncPD);
	//Parameter - Waveform
        //      Testing signal waveform type
        //Parameter - NoiseHz
        //      Value can be 0(without Noize), 50, 60 or 1k Hz
        //Parameter - DC
        //      Range: 100 ~ 2500 mV
        //Parameter - AC
        //      Range: 75 ~ 2500 uV (means 0.75 ~ 25 mV)
        //Parameter - BPM
        //      Range: 30 ~ 300 bpm
        //Parameter - SyncPD
        //      TRUE for testing signal sending synchronized with PD receiving signal
		//Return value:
		//      FALSE for HRS device not connected or Module HRS100 connected

	//Set the Noise Amplitude in "FineTune" Mode
	WHALETEQPPG_SDK_API void SendFineTuneNoiseAmp(double Amp);
	//Parameter - Amp
	//       Default value 0.1 mV
	//       Range 0.01 ~ 0.2 mV

	//Load "Raw Data File" for dynamic playing.
	//When succeed, it would return the sample rate, maximum value, minimum value and total time 
	//with pointer parameters.
	WHALETEQPPG_SDK_API int LoadDynamic(wchar_t* File, double* SampleRate, double* Max, double* Min, double* StopTime);
	//Parameter - File
	//       Full path including file name of "Raw Data File".
	//Parameter - SampleRate
	//       The Sample Rate in "Raw Data File".
	//Parameter - Max
	//       The maximum raw data value in "Raw Data File".
	//Parameter - Min
	//       The minimum raw data value in "Raw Data File".
	//Parameter - StopTime
	//       The total sampling seconds in "Raw Data File".
	//Return value:
	// >0: succeed, the number of sample data loaded
	// -1: Open file failed
	// -2: Can't solve sample rate in Line 1
	// -3: Can't solve sample number in Line 2
	// -4: Raw data file is too large to fit the memory
	// -5: Unsupported API for HRS100 Module

	//Get loaded Raw Data value array.
	WHALETEQPPG_SDK_API double* GetDynamicArray(int* pNum);
	//Parameter - num
        //       The number of raw data values in ValueArray
        //Return value:
        //	     The raw data values array in double type

	//Callback for informing Raw Data playing finish
	typedef void (__stdcall *PlayStopCB)();

	//Callback for real time playing raw data DC and AC value
	typedef void (__stdcall *PlayDynamicCB)(double time, unsigned short DC, unsigned short AC);

	//Start playing loaded Raw Data
	WHALETEQPPG_SDK_API bool PlayDynamic(double Gain, int Offset, PlayStopCB stopCB, PlayDynamicCB playCB, bool SyncPD);
	//Parameter - Gain
	//	Default value is 4095.0 / (Max - Min)
	//Parameter - Offset
	//      Defalut value is "-Min" to normalize the raw data value range
	//Parameter - stopCB
	//		SDK supports "Loop" playing as default mode with stopCB as "(IntPtr)0";
	//      Specified "informing finish callback" to implement "non-Loop" playing mode;
	//Parameter - playCB
	//      Specified real time playing data callback
	//Parameter - SyncPD
    //      TRUE for testing signal sending synchronized with PD receiving signal

	//Change the sample rate of loaded Raw Data for playing
    WHALETEQPPG_SDK_API void ResampleDynamic(double newSampleRate, double* stopTime);
    //Parameter - newSampleRate
    //      New Sample Rate value
    //Parameter - ref stopTime
    //      Updated totally playing time for Sample Rate changed
}