﻿using System;
using System.Collections.Generic;
using System.Text;

using System.Drawing;

using System.Runtime.InteropServices;    // For StructLayout, DllImport
using ZedGraph;
using System.Windows.Forms;
using WhaleTeqPPG_SDK;

namespace WhaleteqPPG_SDK_FineTune
{
    public class Graph : ZedGraphControl
    {
        public delegate void TimeAmplitudeCBDelegate(double time, double amp);
        private static TimeAmplitudeCBDelegate fpTimeAmp; //avoid CB be GC

        public Graph()
        {
            ScrollGrace = 0;
            ScrollMaxX = 0;
            ScrollMaxY = 0;
            ScrollMaxY2 = 0;
            ScrollMinX = 0;
            ScrollMinY = 0;
            ScrollMinY2 = 0;
            GraphPane GP = GraphPane;
            LineItem M = GP.AddCurve("", new PointPairList(), Color.Blue, SymbolType.None);
            LineItem N = GP.AddCurve("", new PointPairList(), Color.DarkViolet, SymbolType.None);
            GP.Title.Text = "PPG Output";
            GP.YAxis.Title.Text = "mV";
            GP.XAxis.Title.Text = "Time (s)";
            double XS = 5;
            Axis X = GraphPane.XAxis;
            X.Scale.Max = XS;
            X.Scale.MajorStep = 1;
            X.Scale.MinorStep = 0.2;
            X.MinorGrid.IsVisible = true;
            X.MinorGrid.DashOff = 1;
            X.MinorGrid.DashOn = 5;
            X.MinorGrid.Color = Color.Pink;
            X.MajorGrid.IsVisible = true;
            X.MajorGrid.DashOff = 0;
            X.MajorGrid.DashOn = 10;
            X.MajorGrid.Color = Color.Pink;

            UpdateScale(1000);

            fpTimeAmp = new TimeAmplitudeCBDelegate(TimeAmplitudeCB);
        }


        public void UpdateScale(int vb)
        {
            YC = vb;

            Axis Y = GraphPane.YAxis;
            Y.Scale.Max = YC + YS;
            Y.Scale.Min = YC - YS;
            Y.Scale.MajorStep = 3;
            Y.Scale.MinorStep = 1.5;
            Y.MinorGrid.IsVisible = true;
            Y.MajorGrid.IsVisible = true;
            Y.MinorGrid.Color = Color.Pink;
            Y.MajorGrid.Color = Color.Pink;
            Y.IsAxisSegmentVisible = false;
            Y.MinorGrid.DashOff = 1;
            Y.MinorGrid.DashOn = 5;
            Y.MajorGrid.DashOff = 0;
            Y.MajorGrid.DashOn = 10;

            ResetGraph();
            Invalidate();
        }

        public void ResetGraph()
        {
            GraphPane.CurveList[0].Clear();
            GraphPane.CurveList[1].Clear();
        }

        private double YC;
        private double YS = 15;

        public void RegisterSDK()
        {
            IntPtr ipTimeAmp = Marshal.GetFunctionPointerForDelegate(fpTimeAmp);

            PPG_SDK.RegisterTimeAmpCB(ipTimeAmp);
        }

        private void RefreshTimerTick(object sender, EventArgs args)
        {
            Invalidate();
        }

        private const int ZBufSize = 64;
        private double[] ZtgBuf = new double[ZBufSize];
        private double[] ZampBuf = new double[ZBufSize];
        private int ZBufIdx = 0;
        private int ZResetCountdown = 0;
        private double lastTg = 0;
        private void TimeAmplitudeCB(double time, double amp)
        {
            double tg = time - ((int)time / 5) * 5;

            if (0 != ZResetCountdown)
            {
                if (0 == --ZResetCountdown)
                    ResetGraph();
            }

            if (ZBufIdx == ZBufSize || tg < lastTg)
            {
                if (tg < lastTg)
                {
                    ZResetCountdown = (ZBufSize / 2);

                }

                for (int i = 0; i < ZBufIdx; i++)
                    GraphPane.CurveList[0].AddPoint(ZtgBuf[i], ZampBuf[i] + YC);

                Invalidate();
                ZBufIdx = 0;
            }

            ZtgBuf[ZBufIdx] = tg;
            ZampBuf[ZBufIdx++] = amp * 2.5;

            lastTg = tg;
        }
    }
}
