﻿using System;
using System.Collections.Generic;
//using System.Linq;
using System.Text;
//using System.Threading.Tasks;

using System.Drawing;

using System.Runtime.InteropServices;    // For StructLayout, DllImport
using ZedGraph;
using WhaleTeqPPG_SDK;

namespace WhaleteqPPG_SDK_PlayRawData
{
    public class LoadGraph : ZedGraphControl
    {
        private double mMax, mMin;

        public LoadGraph()
        {
            ScrollGrace = 0;
            ScrollMaxX = 0;
            ScrollMaxY = 0;
            ScrollMaxY2 = 0;
            ScrollMinX = 0;
            ScrollMinY = 0;
            ScrollMinY2 = 0;

            GraphPane GP = GraphPane;
            LineItem M = GP.AddCurve("", new PointPairList(), Color.Blue, SymbolType.None);
            LineItem N = GP.AddCurve("", new PointPairList(), Color.DarkViolet, SymbolType.None);

            GP.Title.Text = "File";
            GP.YAxis.Title.Text = "Value";
            GP.XAxis.Title.Text = "Time (s)";

            Axis X = GP.XAxis;
            X.Scale.Max = 5;
            X.Scale.Min = 0;
            X.Scale.MajorStep = 1;
            X.Scale.MinorStep = 0.2;
            X.MinorGrid.IsVisible = true;
            X.MinorGrid.DashOff = 1;
            X.MinorGrid.DashOn = 5;
            X.MinorGrid.Color = Color.Pink;
            X.MajorGrid.IsVisible = true;
            X.MajorGrid.DashOff = 0;
            X.MajorGrid.DashOn = 10;
            X.MajorGrid.Color = Color.Pink;

            Axis Y = GP.YAxis;
            Y.MinorGrid.IsVisible = true;
            Y.MajorGrid.IsVisible = true;
            Y.MinorGrid.Color = Color.Pink;
            Y.MajorGrid.Color = Color.Pink;
            Y.IsAxisSegmentVisible = false;

            IsEnableVPan = false;
            IsEnableVZoom = false;
            IsEnableWheelZoom = false;
            IsEnableZoom = false;
        }

        public void UpdateLoad(double max, double min, int rate, double[] loadArray)
        {
            mMax = max;
            mMin = min;

            Axis Y = GraphPane.YAxis;

            Y.Scale.Max = max;
            Y.Scale.Min = min;
            Y.Scale.MajorStep = (int)(max - min) / 6;
            Y.Scale.MinorStep = Y.Scale.MajorStep / 2;

            GraphPane.CurveList[0].Clear();

            int second = 0;
            int sample = 0;
            double time = 0;
            for (int idx = 0; idx < loadArray.Length; idx++, sample++)
            {
                if (sample == rate)
                {
                    ++second;
                    sample = 0;
                }

                time = (double)sample / rate + second;
                GraphPane.CurveList[0].AddPoint(time, loadArray[idx]);
            }

            //PanScale(GraphPane.XAxis, 10, 5);
            //GraphPane.XAxis.Scale.MaxGrace = time;
            //GraphPane.XAxis.Scale.MinGrace = 0;

            Invalidate();
        }

        public void UpdateDynamic(double max, double min, double rate, double[] loadArray)
        {
            if (max != 0.0) //max if 0 for ReSampling
            {
                Axis Y = GraphPane.YAxis;

                //if (min > 0)
                    //min = 0;
                Y.Scale.Max = max;
                Y.Scale.Min = min;

                Y.Scale.MajorStep = (max - min) / 4;
                Y.Scale.MinorStep = Y.Scale.MajorStep / 2;
            }

            GraphPane.CurveList[0].Clear();

            for (int idx = 0; idx < loadArray.Length; idx++)
            {
                double time = (double)idx / rate;
                GraphPane.CurveList[0].AddPoint(time, loadArray[idx]);
            }

            Invalidate();
        }

        public void SetDC(double vb)
        {
            double diff = vb - ((mMax + mMin) / 2);

            Axis Y = GraphPane.YAxis;

            if (diff < 0)
            {
                Y.Scale.Max = mMax;
                Y.Scale.Min = mMin + diff * 2;
            }
            else
            {
                Y.Scale.Max = mMax + diff * 2;
                Y.Scale.Min = mMin;
            }

            Invalidate();
        }

        public void UpdateTime(double time)
        {
            Axis X = GraphPane.XAxis;

            //if (time < X.Scale.Max)
                //return;

            int start = (int)time / 5 * 5;

            X.Scale.Max = start + 5;
            X.Scale.Min = start;

            Invalidate();
        }

        public void ResetGraph()
        {
            GraphPane.CurveList[0].Clear();
            GraphPane.CurveList[1].Clear();
        }
    }
}
